# -*- coding: utf-8 -*-
import socket
import struct

from django.db import models
from django.core.urlresolvers import reverse

class Location(models.Model):
    name = models.CharField(max_length=50)
    region = models.CharField(max_length=50)
    area = models.CharField(max_length=50)
    lat = models.DecimalField(max_digits=10, decimal_places=6)
    lon = models.DecimalField(max_digits=10, decimal_places=6)

    def __unicode__(self):
        return self.name


class RangeManager(models.Manager):
    def find(self, ip):
        """
        Find the smallest range contains the given IP
        """

        number = struct.unpack('!L', socket.inet_aton(ip))[0]
        ranges = self.get_query_set()\
                     .filter(start__lte=number, end__gte=number)\
                     .order_by('end', '-start')\
                     .select_related()
        print 'Ranges:'
        for rang in ranges:
            print rang.country, rang.location
        try:
            return ranges[0]
        except IndexError:
            raise Range.DoesNotExist()


class Range(models.Model):
    start = models.BigIntegerField(max_length=50, db_index=True)
    end = models.BigIntegerField(max_length=50, db_index=True)
    country = models.CharField(max_length=2)
    description = models.CharField(max_length=33)
    location = models.ForeignKey('ipgeo.Location', null=True)

    def __unicode__(self):
        return self.description

    objects = RangeManager()



from StringIO import StringIO
import urllib
from zipfile import ZipFile
from common.progress import Progress

from django.db import transaction
from django.core.management.base import BaseCommand, CommandError

from ipgeo.models import Location, Range

class Command(BaseCommand):
    help = 'Update ipgeo application database'

    @transaction.commit_on_success
    def handle(self, *args, **options):
        print 'Downloading the archive'
        url = 'http://ipgeobase.ru/files/db/Main/geo_files.zip'
        data = urllib.urlopen(url).read()
        zfile = ZipFile(StringIO(data))

        print 'Deleting old objects'
        Location.objects.all().delete()
        Range.objects.all().delete()

        print 'Importing locations'
        for line in zfile.open('cities.txt'):
            keys = ['pk', 'name', 'region', 'area', 'lat', 'lon']
            args = dict(zip(keys, line.decode('cp1251').strip().split('\t')))
            loc = Location.objects.create(**args)

        print 'Importing IP ranges'
        # We know approximate number of range records
        prog = Progress(total=120000)
        for line in zfile.open('cidr_optim.txt'):
            prog.tick()
            keys = ['start', 'end', 'description', 'country', 'location_id']
            args = dict(zip(keys, line.strip().split('\t')))
            if args['location_id'] == '-':
                args['location_id'] = None
            obj = Range.objects.create(**args)



# -*- coding: utf-8
from django.contrib import admin

from ipgeo.models import Location, Range

class LocationAdmin(admin.ModelAdmin):
    list_display = ['name', 'region', 'area']
    search_fields = ['name', 'region', 'area']


class RangeAdmin(admin.ModelAdmin):
    list_display = ['description', 'country', 'location']
    search_fields = ['description', 'location__name']


admin.site.register(Location, LocationAdmin)
admin.site.register(Range, RangeAdmin)
